/******************************************************************************
 *
 *       XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
 *       AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
 *       SOLUTIONS FOR XILINX DEVICES. BY PROVIDING THIS DESIGN, CODE,
 *       OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
 *       APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
 *       THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
 *       AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
 *       FOR YOUR IMPLEMENTATION. XILINX EXPRESSLY DISCLAIMS ANY
 *       WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
 *       IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
 *       REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
 *       INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *       FOR A PARTICULAR PURPOSE.
 *
 *       (c) Copyright 2007 Xilinx Inc.
 *       All rights reserved.
 *
 *****************************************************************************/
/*****************************************************************************/
/**
 * @file xusb_haldemo.c      
 *
 * This file contains haldemo related functions.      
 *
 * @note	None.
 *
 * <pre>   
 * MODIFICATION HISTORY:
 *
 * Ver   Who  Date	Changes
 * ----- ---- -----------------------------------------------------------------
 * 1.00a hvm  2/22/07 First release
 *
 * </pre>
 *****************************************************************************/
/***************************** Include Files *********************************/

#include "xusb.h"
#include "xintc.h"
#include "xusb_endpointrdwr.h"        
#include <xmk.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <os_config.h>
#include <sys/process.h>
#include <pthread.h>
#include <sys/decls.h>

/************************** Constant Definitions *****************************/

#define USB_DEVICE_ID		XPAR_USB_0_DEVICE_ID
#define INTC_DEVICE_ID		XPAR_OPB_INTC_0_DEVICE_ID
#define USB_INTR		XPAR_OPB_INTC_0_OPB_USB2_DEVICE_0_USB_IRPT_INTR


#define USB_RESET_VALUE		1
#define USB_SUSPEND_VALUE	2
#define USB_DISCONNECT_VALUE	3
#define USB_CONFIG_VALUE	4

#define MAX_BUF_SIZE		32768

#define XUSB_EPRW		TRUE
/************************** Variable Definitions *****************************/

XUsb UsbInstance;		/* The instance of the USB device */
XUsb_Config	*UsbConfigPtr;	/* Instance of the USB config structure */

XIntc InterruptController;	/* Instance of the Interrupt Controller */

sem_t UsbEventSem;

/*
 *  Maximum control size
 */
extern u16	MaxControlSize;


typedef struct {
	sem_t Buf0Sem;
	sem_t Buf1Sem;
} EpcsSem;

EpcsSem Epcs[8];

int RunFlag;
/*
 * USB Event value that caused the UsbEventSem to fire
 */
int UsbEventValue;


pthread_t Loop14Tid;
pthread_t Loop25Tid;
pthread_t Loop36Tid;
pthread_t UsbEnum_tid;

u8 EpData14[MAX_BUF_SIZE];
u8 EpData25[MAX_BUF_SIZE];
u8 EpData36[MAX_BUF_SIZE];

/************************** Function Prototypes *******************************/

void StartupUserTasks(void);
void ShutdownUserTasks(void);
void WakeupAndStop(XUsb *InstancePtr);

/*
 * The Tasks
 */
void Loop14(void * pdata);
void Loop25(void * pdata);
void Loop36(void * pdata);
void UsbEnumerated(void *pdata);
int USBControl( void *dummy);


/*****************************************************************************/
/**
 * This main function starts the xilinx micro kernel.
 *
 *
 * @param	None.
 *
 * @return	None.
 *
 * @note	None.
 *
 *****************************************************************************/
int main ()
{
    xilkernel_main ();
    return 0;
}

/**********dummy function*************************/
void MassStorageReset(XUsb * InstancePtr)
{
	return;
}

/*****************************************************************************/
/**
 * This function starts the USB application. This is called from OS. the
 * function is attached to the kernel init.
 *
 *
 * @param	None.
 *
 * @return
 *		- XST_SUCCESS if successful.
 *		- XST_FAILURE if test fails.
 * @note	None.
 *
 *****************************************************************************/
int USBControl (void *dummy)
{
	int Status;
	int tasks_running = 0;
	u8 Index;

	/*
	 * Initialize the USB driver.
	 */
	UsbConfigPtr = XUsb_LookupConfig(USB_DEVICE_ID);
	if (NULL == UsbConfigPtr) {
		return XST_FAILURE;
	}

	/*
	 * We are passing the physical base address as the third argument
	 * because the physical and virtual base address are the same in our
	 * example. For systems that support virtual memory, the third
	 * argument needs to be the virtual base address.
	 */
	Status = XUsb_CfgInitialize(&UsbInstance,
				       UsbConfigPtr,
				       UsbConfigPtr->BaseAddress);
	if (Status != XST_SUCCESS ) {
		return XST_FAILURE;
	}

	/*
	 * Initialize the USB instance as required for the mass storage
	 * application.
	 */
	InitUsbInterface(&UsbInstance);

	/*
	 * Set our function address to 0 which is the unenumerated state.
	 */
	Status = XUsb_SetDeviceAddress(&UsbInstance, 0);
	if (Status != XST_SUCCESS ) {
		return XST_FAILURE;
	}

	/*
	 * Setup the interrupt handlers.
	 */
	XUsb_IntrSetHandler(&UsbInstance, (void *)UsbIfIntrHandler,
				&UsbInstance);

	XUsb_EpSetHandler(&UsbInstance, XUSB_EP_NUMBER_ZERO,
			(XUsb_EpHandlerFunc *)Ep0IntrHandler, &UsbInstance);

	XUsb_EpSetHandler(&UsbInstance, XUSB_EP_NUMBER_ONE,
			(XUsb_EpHandlerFunc *)Ep1IntrHandler, &UsbInstance);

	XUsb_EpSetHandler(&UsbInstance, XUSB_EP_NUMBER_TWO,
			(XUsb_EpHandlerFunc *)Ep2IntrHandler, &UsbInstance);

	XUsb_EpSetHandler(&UsbInstance, XUSB_EP_NUMBER_THREE,
			(XUsb_EpHandlerFunc *)Ep3IntrHandler, &UsbInstance);

	XUsb_EpSetHandler(&UsbInstance, XUSB_EP_NUMBER_FOUR,
			(XUsb_EpHandlerFunc *)Ep4IntrHandler, &UsbInstance);

	XUsb_EpSetHandler(&UsbInstance, XUSB_EP_NUMBER_FIVE,
			(XUsb_EpHandlerFunc *)Ep5IntrHandler, &UsbInstance);

	XUsb_EpSetHandler(&UsbInstance, XUSB_EP_NUMBER_SIX,
			(XUsb_EpHandlerFunc *)Ep6IntrHandler, &UsbInstance);

	XUsb_EpSetHandler(&UsbInstance, XUSB_EP_NUMBER_SEVEN,
			(XUsb_EpHandlerFunc *)Ep7IntrHandler, &UsbInstance);

	/*
	 * Setup the Usb Event Semaphore
	 */
	sem_init(&UsbEventSem, 1, 0);

	/*
	 * Create the Semaphores used to control buffer availability
	 * between foreground and the interrupt handler
	 */
	for(Index = 0; Index < 8; Index++) {
		sem_init(&(Epcs[Index].Buf0Sem), 1, 0);
		sem_init(&(Epcs[Index].Buf1Sem), 1, 0);
	}

	/*
	 * Setup the interrupt system.
	 */
	Status = SetupInterruptSystem(&UsbInstance);
	if (Status != XST_SUCCESS) {
		return XST_FAILURE;
	}

	/*
	 * Enable the interrupts.
	 */
	XUsb_IntrEnable(&UsbInstance, XUSB_STATUS_GLOBAL_INTR_MASK |
				XUSB_STATUS_RESET_MASK |
				XUSB_STATUS_SUSPEND_MASK |
				XUSB_STATUS_SETUP_PACKET_MASK |
				XUSB_STATUS_DISCONNECT_MASK |
				XUSB_STATUS_FIFO_BUFF_RDY_MASK |
				XUSB_STATUS_FIFO_BUFF_FREE_MASK |
				XUSB_STATUS_INTR_BUFF_COMP_ALL_MASK);

	XUsb_Start(&UsbInstance);

	/*
	 * Set the device configuration to unenumerated state.
	 */
	UsbInstance.DeviceConfig.CurrentConfiguration = 0;

	pthread_create(&UsbEnum_tid, NULL, (void *)UsbEnumerated, &UsbInstance);

	while(1) {

		sem_wait(&UsbEventSem);

		/*
		 * Act on the event received.
		 */
		switch(UsbEventValue)
		{
			case USB_DISCONNECT_VALUE:
				printf("Disconnect\r\n");
				if(tasks_running) {
					printf("Shutdown\r\n");
					ShutdownUserTasks();
					tasks_running = 0;
				}
				break;

			case USB_RESET_VALUE:
				printf("USB Reset\r\n");
				if(tasks_running) {
					printf("Shutdown\r\n");
					ShutdownUserTasks();
					tasks_running = 0;
				}
				break;

			case USB_SUSPEND_VALUE:
				if(tasks_running) {
					printf("Shutdown\r\n");
					ShutdownUserTasks();
					tasks_running = 0;
				}
				break;

			case USB_CONFIG_VALUE:
				if(tasks_running == 0) {
					printf("C1 Startup\r\n");
					StartupUserTasks();
					tasks_running = 1;
					RunFlag = 1;
				}
				break;

			default:
				printf("USBControl: Unknown event\n");
				printf("UsbEventValue=%x\r\n",
				UsbEventValue);
				break;
		}
	}

	return Status;

}

/*****************************************************************************/
/**
 * This is the USB initialization function. This example initializes the device
 * for Windows Application. The following configuration is done.
 *	- EP0 : CONTROL end point, Bidirectional, Packet size 64 bytes.
 *	- EP1 : NON_ISOCHRONOUS, BULK_OUT, packet size 512 bytes.
 *	- EP2 : NON_ISOCHRONOUS, BULK_OUT, packet size 512 bytes
 *	- EP3 : NON_ISOCHRONOUS, BULK_OUT, packet size 512 bytes.
 *	- EP4 : NON_ISOCHRONOUS, BULK_IN, packet size 512 bytes
 *	- EP5 : NON_ISOCHRONOUS, BULK_IN, packet size 512 bytes.
 *	- EP6 : NON_ISOCHRONOUS, BULK_IN, packet size 512 bytes
 *	- EP7 : NON_ISOCHRONOUS, INTERRUPT_IN, packet size 16 bytes
 *
 *
 * @param	InstancePtr is a pointer to the XUsb instance.
 *
 * @return	None.
 *
 * @note	None.
 *
 ******************************************************************************/
void InitUsbInterface(XUsb *InstancePtr)
{

	XUsb_DeviceConfig	DeviceConfig;
	u8 Index;

	/*
	 * Setup Endpoint 0.
	 */
	DeviceConfig.Ep[XUSB_EP_NUMBER_ZERO].RamBase = 0x30;
	DeviceConfig.Ep[XUSB_EP_NUMBER_ZERO].Size = 0x40;
	DeviceConfig.Ep[XUSB_EP_NUMBER_ZERO].EpType = 0;
	DeviceConfig.Ep[XUSB_EP_NUMBER_ZERO].OutIn = XUSB_EP_DIRECTION_OUT;

	/*
	 * Setup end points 1, 2 and 3 as BULK OUT type and end point maximum
	 * size as 512 bytes.
	 */
	DeviceConfig.Ep[XUSB_EP_NUMBER_ONE].RamBase = 0x100;
	DeviceConfig.Ep[XUSB_EP_NUMBER_ONE].Size = 0x200;
	DeviceConfig.Ep[XUSB_EP_NUMBER_ONE].EpType = 0;
	DeviceConfig.Ep[XUSB_EP_NUMBER_ONE].OutIn = XUSB_EP_DIRECTION_OUT;

	DeviceConfig.Ep[XUSB_EP_NUMBER_TWO].RamBase = 0x200;
	DeviceConfig.Ep[XUSB_EP_NUMBER_TWO].Size = 0x200;
	DeviceConfig.Ep[XUSB_EP_NUMBER_TWO].EpType = 0;
	DeviceConfig.Ep[XUSB_EP_NUMBER_TWO].OutIn = XUSB_EP_DIRECTION_OUT;

	DeviceConfig.Ep[XUSB_EP_NUMBER_THREE].RamBase = 0x300;
	DeviceConfig.Ep[XUSB_EP_NUMBER_THREE].Size = 0x200;
	DeviceConfig.Ep[XUSB_EP_NUMBER_THREE].EpType = 0;
	DeviceConfig.Ep[XUSB_EP_NUMBER_THREE].OutIn = XUSB_EP_DIRECTION_OUT;

	/*
	 * Set up the end points 4, 5 and 6 as BULK_IN type and end point
	 * maximum size as 512 bytes.
	 */
	DeviceConfig.Ep[XUSB_EP_NUMBER_FOUR].RamBase = 0x400;
	DeviceConfig.Ep[XUSB_EP_NUMBER_FOUR].Size = 0x200;
	DeviceConfig.Ep[XUSB_EP_NUMBER_FOUR].EpType = 0;
	DeviceConfig.Ep[XUSB_EP_NUMBER_FOUR].OutIn = XUSB_EP_DIRECTION_IN;

	DeviceConfig.Ep[XUSB_EP_NUMBER_FIVE].RamBase = 0x500;
	DeviceConfig.Ep[XUSB_EP_NUMBER_FIVE].Size = 0x200;
	DeviceConfig.Ep[XUSB_EP_NUMBER_FIVE].EpType = 0;
	DeviceConfig.Ep[XUSB_EP_NUMBER_FIVE].OutIn = XUSB_EP_DIRECTION_IN;

	DeviceConfig.Ep[XUSB_EP_NUMBER_SIX].RamBase = 0x600;
	DeviceConfig.Ep[XUSB_EP_NUMBER_SIX].Size = 0x200;
	DeviceConfig.Ep[XUSB_EP_NUMBER_SIX].EpType = 0;
	DeviceConfig.Ep[XUSB_EP_NUMBER_SIX].OutIn = XUSB_EP_DIRECTION_IN;

	/*
	 * Setup the end point 7 as INTERRUPT_IN interrupt mode and maximum size
	 * as  16 bytes.
	 */
	DeviceConfig.Ep[XUSB_EP_NUMBER_SEVEN].RamBase = 0x700;
	DeviceConfig.Ep[XUSB_EP_NUMBER_SEVEN].Size = 0x010;
	DeviceConfig.Ep[XUSB_EP_NUMBER_SEVEN].EpType = 0;
	DeviceConfig.Ep[XUSB_EP_NUMBER_SEVEN].OutIn = XUSB_EP_DIRECTION_IN;

	DeviceConfig.NumEndpoints = 8; 


	/*
	 * Initialize the device configuration.
	 */
	XUsb_ConfigureDevice(InstancePtr, &DeviceConfig);

	XUsb_EpEnable(InstancePtr, XUSB_EP_NUMBER_ZERO);
	XUsb_EpEnable(InstancePtr, XUSB_EP_NUMBER_ONE);
	XUsb_EpEnable(InstancePtr, XUSB_EP_NUMBER_TWO);
	XUsb_EpEnable(InstancePtr, XUSB_EP_NUMBER_THREE);
	XUsb_EpEnable(InstancePtr, XUSB_EP_NUMBER_FOUR);
	XUsb_EpEnable(InstancePtr, XUSB_EP_NUMBER_FIVE);
	XUsb_EpEnable(InstancePtr, XUSB_EP_NUMBER_SIX);
	XUsb_EpEnable(InstancePtr, XUSB_EP_NUMBER_SEVEN);

	/*
	 * Make the OUT end points ready to receive data.
	 */
	for (Index = XUSB_EP_NUMBER_ONE; Index < XUSB_EP_NUMBER_FOUR; Index++) {

		XUsb_mWriteReg(InstancePtr->Config.BaseAddress,
			XUSB_BUFFREADY_OFFSET, 1 << Index);

		InstancePtr->DeviceConfig.Ep[Index].Buffer0Ready = 1;

		XUsb_mWriteReg(InstancePtr->Config.BaseAddress,
			XUSB_BUFFREADY_OFFSET, (1 <<
			(Index + XUSB_STATUS_EP_BUFF2_SHIFT)));

		InstancePtr->DeviceConfig.Ep[Index].Buffer1Ready = 1;
	}
	MaxControlSize = 64;

	/*
	 * Store the actual RAM address offset in the device structure, so as to
	 * avoid the multiplication during processing.
	 */
	for (Index = XUSB_EP_NUMBER_ONE; Index < XUSB_MAX_ENDPOINTS; Index++) {

		InstancePtr->DeviceConfig.Ep[Index].RamBase <<= 2 ;
	}

}

/*****************************************************************************/
/**
 * This function is the interrupt handler for the USB windows application.
 *
 *
 * @param	CallBackRef is the callback reference passed from the interrupt
 *		handler, which in our case is a pointer to the driver instance.
 * @param	IntrStatus is a bit mask indicating pending interrupts.
 *
 * @return	None.
 *
 * @note	None.
 *
 ******************************************************************************/
void UsbIfIntrHandler(void *CallBackRef, u32 IntrStatus)
{

	XUsb *InstancePtr;

	InstancePtr = (XUsb *)CallBackRef;

	if (IntrStatus & XUSB_STATUS_RESET_MASK){
		/*
		 * Set device address to 0 (required by reset).
		 */
		UsbEventValue = USB_RESET_VALUE;
		sem_post(&UsbEventSem);
		if (InstancePtr->DeviceConfig.CurrentConfiguration == 1) {
			InstancePtr->DeviceConfig.CurrentConfiguration = 0;
			InitUsbInterface(InstancePtr);
			XUsb_SetDeviceAddress(InstancePtr, 0);
		}
		XUsb_IntrDisable(InstancePtr, XUSB_STATUS_RESET_MASK);
		XUsb_IntrEnable(InstancePtr, (XUSB_STATUS_DISCONNECT_MASK |
					XUSB_STATUS_SUSPEND_MASK));
		WakeupAndStop(InstancePtr);

	}
	if (IntrStatus & XUSB_STATUS_SUSPEND_MASK){
		if (RunFlag == 1){
			UsbEventValue = USB_SUSPEND_VALUE;
			sem_post(&UsbEventSem);
		}
		XUsb_IntrDisable(InstancePtr, XUSB_STATUS_SUSPEND_MASK);
		XUsb_IntrEnable(InstancePtr, (XUSB_STATUS_RESET_MASK |
					XUSB_STATUS_DISCONNECT_MASK));
		WakeupAndStop(InstancePtr);
	}
	if (IntrStatus & XUSB_STATUS_DISCONNECT_MASK){
		if (InstancePtr->DeviceConfig.CurrentConfiguration == 1) {
			InstancePtr->DeviceConfig.CurrentConfiguration = 0;
			InitUsbInterface(InstancePtr);
			XUsb_SetDeviceAddress(InstancePtr, 0);
			UsbEventValue = USB_DISCONNECT_VALUE;
			sem_post(&UsbEventSem);
		}
		XUsb_IntrDisable(InstancePtr,
				XUSB_STATUS_DISCONNECT_MASK);
		XUsb_IntrEnable(InstancePtr, (XUSB_STATUS_RESET_MASK |
					XUSB_STATUS_SUSPEND_MASK));
		WakeupAndStop(InstancePtr);
	}
}

/*****************************************************************************/
/**
 * This function is the interrupt handler for the USB End point Zero events.
 *
 * @param	CallBackRef is the callback reference passed from the interrupt.
 *		handler, which in our case is a pointer to the driver instance.
 * @param	EpNum is the end point number.
 * @param	IntrStatus is a bit mask indicating pending interrupts.
 *
 * @return	None.
 *
 * @note	EpNum is not used in this function as the handler is attached
 *		specific to end point zero. This parameter is useful when a
 *		single handler is used for processing all end point interrupts.
 *
 ******************************************************************************/
void Ep0IntrHandler(void *CallBackRef, u8 EpNum, u32 IntrStatus)
{

	XUsb *InstancePtr;

	InstancePtr = (XUsb *)CallBackRef;

	/*
	 * Process the end point zero buffer interrupt.
	 */
	if (IntrStatus & XUSB_BUFFREADY_EP0_BUFF_MASK){
		if (IntrStatus & XUSB_STATUS_SETUP_PACKET_MASK) {
			/*
			 * Received a setup packet. Execute the chapter 9
			 * command.
			 */
			XUsb_IntrEnable(InstancePtr, 
					(XUSB_STATUS_DISCONNECT_MASK |
					XUSB_STATUS_SUSPEND_MASK | 
					XUSB_STATUS_RESET_MASK));
			Chapter9(InstancePtr);
		} else if (IntrStatus & XUSB_STATUS_FIFO_BUFF_RDY_MASK) {
				EP0ProcessOutToken(InstancePtr);
		} else if (IntrStatus & XUSB_STATUS_FIFO_BUFF_FREE_MASK) {
				EP0ProcessInToken(InstancePtr);
		}
	}
}

/*****************************************************************************/
/**
 * This function is the interrupt handler for the USB End point one events.
 *
 * @param	CallBackRef is the callback reference passed from the interrupt
 *		handler, which in our case is a pointer to the driver instance.
 * @param	EpNum is the end point number.
 * @param	IntrStatus is a bit mask indicating pending interrupts.
 *
 * @return	None.
 *
 * @note	EpNum is not used in this function as the handler is attached
 *		specific to end point one. This parameter is useful when a
 *		single handler is used for processing all end point interrupts.
 *
 ******************************************************************************/
void Ep1IntrHandler(void *CallBackRef, u8 EpNum, u32 IntrStatus)
{

	XUsb *InstancePtr;

	InstancePtr = (XUsb *)CallBackRef;

	/*
	 * Process the End point 1 interrupts.
	 */
	if (IntrStatus & XUSB_BUFFREADY_EP1_BUFF1_MASK) {
		InstancePtr->DeviceConfig.Ep[1].Buffer0Ready = 0;
		sem_post(&Epcs[1].Buf0Sem);

	}

	if (IntrStatus & XUSB_BUFFREADY_EP1_BUFF2_MASK) {
		InstancePtr->DeviceConfig.Ep[1].Buffer1Ready = 0;
		sem_post(&Epcs[1].Buf1Sem);

	}

}

/*****************************************************************************/
/**
 * This function is the interrupt handler for the USB End point two events.
 *
 * @param	CallBackRef is the callback reference passed from the interrupt
 *		handler, which in our case is a pointer to the driver instance.
 * @param	EpNum is the end point number.
 * @param	IntrStatus is a bit mask indicating pending interrupts.
 *
 * @return	None.
 *
 * @note	EpNum is not used in this function as the handler is attached
 *		specific to end point two. This parameter is useful when a
 *		single handler is used for processing all end point interrupts.
 *
 ******************************************************************************/
void Ep2IntrHandler(void *CallBackRef, u8 EpNum, u32 IntrStatus)
{

	XUsb *InstancePtr;

	InstancePtr = (XUsb *)CallBackRef;

	/*
	 * Process end point 2 interrupts.
	 */
	if (IntrStatus & XUSB_BUFFREADY_EP2_BUFF1_MASK) {
		InstancePtr->DeviceConfig.Ep[2].Buffer0Ready = 0;
		sem_post(&Epcs[2].Buf0Sem);
	}
	if(IntrStatus & XUSB_BUFFREADY_EP2_BUFF2_MASK) {
		InstancePtr->DeviceConfig.Ep[2].Buffer1Ready = 0;
		sem_post(&Epcs[2].Buf1Sem);
	}
}

/*****************************************************************************/
/**
 * This function is the interrupt handler for the USB End point three events.
 *
 * @param	CallBackRef is the callback reference passed from the interrupt
 *		handler, which in our case is a pointer to the driver instance.
 * @param	EpNum is the end point number.
 * @param	IntrStatus is a bit mask indicating pending interrupts.
 *
 * @return	None.
 *
 * @note	EpNum is not used in this function as the handler is attached
 *		specific to end point two. This parameter is useful when a
 *		single handler is used for processing all end point interrupts.
 *
 ******************************************************************************/
void Ep3IntrHandler(void *CallBackRef, u8 EpNum, u32 IntrStatus)
{

	XUsb *InstancePtr;

	InstancePtr = (XUsb *)CallBackRef;

	/*
	 * Process end point 3 interrupts.
	 */
	if (IntrStatus & XUSB_BUFFREADY_EP3_BUFF1_MASK) {
		InstancePtr->DeviceConfig.Ep[3].Buffer0Ready = 0;
		sem_post(&Epcs[3].Buf0Sem);
	}
	if(IntrStatus & XUSB_BUFFREADY_EP3_BUFF2_MASK) {
		InstancePtr->DeviceConfig.Ep[3].Buffer1Ready = 0;
		sem_post(&Epcs[3].Buf1Sem);
	}
}
/*****************************************************************************/
/**
 * This function is the interrupt handler for the USB End point four events.
 *
 * @param	CallBackRef is the callback reference passed from the interrupt
 *		handler, which in our case is a pointer to the driver instance.
 * @param	EpNum is the end point number.
 * @param	IntrStatus is a bit mask indicating pending interrupts.
 *
 * @return	None.
 *
 * @note	EpNum is not used in this function as the handler is attached
 *		specific to end point two. This parameter is useful when a
 *		single handler is used for processing all end point interrupts.
 *
 ******************************************************************************/
void Ep4IntrHandler(void *CallBackRef, u8 EpNum, u32 IntrStatus)
{

	XUsb *InstancePtr;

	InstancePtr = (XUsb *)CallBackRef;

	/*
	 * Process end point 4 interrupts.
	 */
	if (IntrStatus & XUSB_BUFFREADY_EP4_BUFF1_MASK) {
		InstancePtr->DeviceConfig.Ep[4].Buffer0Ready = 0;
		sem_post(&Epcs[4].Buf0Sem);
	}
	if(IntrStatus & XUSB_BUFFREADY_EP4_BUFF2_MASK) {
		InstancePtr->DeviceConfig.Ep[4].Buffer1Ready = 0;
		sem_post(&Epcs[4].Buf1Sem);
	}
}
/*****************************************************************************/
/**
 * This function is the interrupt handler for the USB End point five events.
 *
 * @param	CallBackRef is the callback reference passed from the interrupt
 *		handler, which in our case is a pointer to the driver instance.
 * @param	EpNum is the end point number.
 * @param	IntrStatus is a bit mask indicating pending interrupts.
 *
 * @return	None.
 *
 * @note	EpNum is not used in this function as the handler is attached
 *		specific to end point two. This parameter is useful when a
 *		single handler is used for processing all end point interrupts.
 *
 ******************************************************************************/
void Ep5IntrHandler(void *CallBackRef, u8 EpNum, u32 IntrStatus)
{

	XUsb *InstancePtr;

	InstancePtr = (XUsb *)CallBackRef;

	/*
	 * Process end point 5 interrupts.
	 */
	if (IntrStatus & XUSB_BUFFREADY_EP5_BUFF1_MASK) {
		InstancePtr->DeviceConfig.Ep[5].Buffer0Ready = 0;
		sem_post(&Epcs[5].Buf0Sem);
	}
	if(IntrStatus & XUSB_BUFFREADY_EP5_BUFF2_MASK) {
		InstancePtr->DeviceConfig.Ep[5].Buffer1Ready = 0;
		sem_post(&Epcs[5].Buf1Sem);
	}
}
/*****************************************************************************/
/**
 * This function is the interrupt handler for the USB End point two events.
 *
 * @param	CallBackRef is the callback reference passed from the interrupt
 *		handler, which in our case is a pointer to the driver instance.
 * @param	EpNum is the end point number.
 * @param	IntrStatus is a bit mask indicating pending interrupts.
 *
 * @return	None.
 *
 * @note	EpNum is not used in this function as the handler is attached
 *		specific to end point two. This parameter is useful when a
 *		single handler is used for processing all end point interrupts.
 *
 ******************************************************************************/
void Ep6IntrHandler(void *CallBackRef, u8 EpNum, u32 IntrStatus)
{

	XUsb *InstancePtr;

	InstancePtr = (XUsb *)CallBackRef;

	/*
	 * Process end point 6 interrupts.
	 */
	if (IntrStatus & XUSB_BUFFREADY_EP6_BUFF1_MASK) {
		InstancePtr->DeviceConfig.Ep[6].Buffer0Ready = 0;
		sem_post(&Epcs[6].Buf0Sem);
	}
	if(IntrStatus & XUSB_BUFFREADY_EP6_BUFF2_MASK) {
		InstancePtr->DeviceConfig.Ep[6].Buffer1Ready = 0;
		sem_post(&Epcs[6].Buf1Sem);
	}
}

/*****************************************************************************/
/**
 * This function is the interrupt handler for the USB End point seven events.
 *
 * @param	CallBackRef is the callback reference passed from the interrupt
 *		handler, which in our case is a pointer to the driver instance.
 * @param	EpNum is the end point number.
 * @param	IntrStatus is a bit mask indicating pending interrupts.
 *
 * @return	None.
 *
 * @note	EpNum is not used in this function as the handler is attached
 *		specific to end point two. This parameter is useful when a
 *		single handler is used for processing all end point interrupts.
 *
 ******************************************************************************/
void Ep7IntrHandler(void *CallBackRef, u8 EpNum, u32 IntrStatus)
{

	XUsb *InstancePtr;

	InstancePtr = (XUsb *)CallBackRef;

	/*
	 * Process end point 7 interrupts.
	 */
	if (IntrStatus & XUSB_BUFFREADY_EP7_BUFF1_MASK) {
		InstancePtr->DeviceConfig.Ep[7].Buffer0Ready = 0;
		sem_post(&Epcs[7].Buf0Sem);
	}
	if(IntrStatus & XUSB_BUFFREADY_EP7_BUFF2_MASK) {
		InstancePtr->DeviceConfig.Ep[7].Buffer1Ready = 0;
		sem_post(&Epcs[7].Buf1Sem);
	}
}

/******************************************************************************/
/**
*
* This function sets up the interrupt system such that interrupts can occur
* for the USB. This function is application specific since the actual
* system may or may not have an interrupt controller. The USB could be
* directly connected to a processor without an interrupt controller.  The
* user should modify this function to fit the application.
*
* @param	InstancePtr contains a pointer to the instance of the USB
*		component, which is going to be connected to the interrupt
*		controller.
*
* @return
*		- XST_SUCCESS if successful.
*		- XST_FAILURE. if it fails.
*
* @note		None
*
*******************************************************************************/
static int SetupInterruptSystem(XUsb *InstancePtr)
{
	int Status;

	/*
	 * Initialize the interrupt controller driver.
	 */
	Status = XIntc_Initialize(&InterruptController, INTC_DEVICE_ID);
	if (Status != XST_SUCCESS){
		return XST_FAILURE;
	}


	/*
	 * Connect a device driver handler that will be called when an interrupt
	 * for the USB device occurs.
	 */
	Status = XIntc_Connect(&InterruptController, USB_INTR,
			    (XInterruptHandler)XUsb_IntrHandler,
			    (void *)InstancePtr);
	if (Status != XST_SUCCESS){
		return XST_FAILURE;
	}

	/*
	 * Start the interrupt controller such that interrupts are enabled for
	 * all devices that cause interrupts, specific real mode so that
	 * the USB can cause interrupts through the interrupt controller.
	 */
	Status = XIntc_Start(&InterruptController, XIN_REAL_MODE);
	if (Status != XST_SUCCESS){
		return XST_FAILURE;
	}

	/*
	 * Enable the interrupt for the USB.
	 */
	XIntc_Enable(&InterruptController, USB_INTR);

	/*
	 * Enable the Interrupts for the MicroBlaze Processor.
	 */
	microblaze_enable_interrupts();

	return XST_SUCCESS;
}

/******************************************************************************/
/**
*
* This function reads the received data from the given end point. 
*
* @param	InstancePtr contains a pointer to the instance of the USB
*		component, which is going to be connected to the interrupt
*		controller.
* @param	EpNum is the endpoint number 
* @param	NoBlock makes the function blocking or non-blocking.
* @param	BufPtr is the buffer pointer where the data is to be read into.
* @param	Length is the amount of data to be read in bytes
* @return
*		- XST_SUCCESS if successful.
*		- XST_FAILURE. if it fails.
*
* @note		None
*
*******************************************************************************/
int USBRead(XUsb *InstancePtr, int EpNum, int NoBlock, char *BufPtr, int Length)
{
	int Results = 0;
	int BytesToRead;

	if(InstancePtr->DeviceConfig.CurrentConfiguration == 0) {
		Results = -1;
		return(Results);
	}

	/*
	 * See if this is an OUT endpoint (in other words, read is valid)
	 */
	if(EpNum > 3) {
		/*
		 * Endpoints greater than 3 are IN
		 */
		Results = -1;
		return(Results);
	}
	while(Length) {
		if(RunFlag == 0) {
			Results = 0;
			break;
		}

		if (Length > InstancePtr->DeviceConfig.Ep[EpNum].Size ) {
			BytesToRead =	
				InstancePtr->DeviceConfig.Ep[EpNum].Size;
		} else {
			BytesToRead = Length;
		}

		if ((XUsb_EpDataRecv(InstancePtr, EpNum, BufPtr,
					BytesToRead) != XST_SUCCESS)) {
			if(RunFlag == 0) {
				Results = -1;
				break;
			}

			if(NoBlock == 0) {
				/*
				 * Wait until we get one
				 */
				if (InstancePtr->DeviceConfig.Ep[EpNum].
					CurBufNum == 0){
					sem_wait(&Epcs[EpNum].Buf0Sem);
					if(RunFlag == 0) {
						Results = -1;
						break;
					}
				} else {
					sem_wait(&Epcs[EpNum].Buf1Sem);
					if(RunFlag == 0) {
						Results = -1;
						break;
					}
				}
				if(InstancePtr->DeviceConfig.
					CurrentConfiguration == 0) {
					Results = 0;
					break;
				}
			}else {
				/*
				 * User does not want to block so just return
				 */
				Results = 0;
				break;
			}
		}

		Results += BytesToRead;
		Length -= BytesToRead;
		BufPtr += BytesToRead;
	}

	return(Results);
}

/******************************************************************************/
/**
*
* This function writes the data for the given end point for transmission 
*
* @param	InstancePtr contains a pointer to the instance of the USB
*		component, which is going to be connected to the interrupt
*		controller.
* @param	EpNum is the endpoint number 
* @param	NoBlock makes the function blocking or non-blocking.
* @param	BufPtr is the buffer pointer where the data is to be written.
* @param	Length is the amount of data to be written in bytes
* @return
*		- XST_SUCCESS if successful.
*		- XST_FAILURE. if it fails.
*
* @note		None
*
*******************************************************************************/
int USBWrite(XUsb *InstancePtr, int EpNum, int NoBlock, char *BufPtr, 
		int Length)
{
	int Results = 0;
	int BytesToWrite;

	if(InstancePtr->DeviceConfig.CurrentConfiguration == 0) {
		Results = -1;
		return(Results);
	}

	/*
	 * See if this is an IN endpoint (in other words, write is valid)
	 */
	if(EpNum < 4) {
		/*
		 * endpoints greater less than 4 are OUT
		 */
		Results = -1;
		return(Results);
	}

	while(Length) {
		if(RunFlag == 0) {
			Results = 0;
			break;
		}

		if (Length > InstancePtr->DeviceConfig.Ep[EpNum].Size ) {
			BytesToWrite =
				InstancePtr->DeviceConfig.Ep[EpNum].Size;
		} else {
			BytesToWrite = Length;
		}

		if ((XUsb_EpDataSend(InstancePtr, EpNum, BufPtr,
					BytesToWrite) != XST_SUCCESS)) {
			if(RunFlag== 0) {
				Results = -1;
				break;
			}

			if(NoBlock == 0) {
				/*
				 * Wait until we get one
				 */
				if (InstancePtr->DeviceConfig.Ep[EpNum].
					CurBufNum == 0){
					sem_wait(&Epcs[EpNum].Buf0Sem);
				} else {
					sem_wait(&Epcs[EpNum].Buf1Sem);
				}
				if(RunFlag == 0) {
					Results = 0;
					break;
				}
			}else {
				/*
				 * User does not want to block so just return
				 */
				Results = 0;
				break;
			}
		}

		Results += BytesToWrite;
		Length -= BytesToWrite;
		BufPtr += BytesToWrite;
	}

	return(Results);
}
/******************************************************************************/
/**
*
* This function starts the user tasks
*
* @param	None
*
* @return	None
*
* @note		None
*
*******************************************************************************/
void StartupUserTasks(void)
{
	int Err;

	RunFlag = 1;

	/*
	 * Start the User Tasks to process requests
	 */
	Err = pthread_create(&Loop14Tid, NULL, (void *)Loop14, &UsbInstance);
	if(Err != 0) {
		printf("Could not create Loop14 task\n");
	} else {
		printf("loop14_tid = %d\n", Loop14Tid);
	}

	Err = pthread_create(&Loop25Tid, NULL, (void *)Loop25, &UsbInstance);
	if(Err != 0) {
		printf("Could not create Loop25 task\n");
	}else {
		printf("loop25_tid = %d\n", Loop25Tid);
	}

	Err = pthread_create(&Loop36Tid, NULL, (void *)Loop36, &UsbInstance);
	if(Err != 0) {
		printf("Could not create Loop36 task\n");
	}else {
		printf("loop36_tid = %d\n", Loop36Tid);
	}
}

/******************************************************************************/
/**
*
* This function is the thread which receives data on end point 1 and 
* transmitting on end point 4
*
* @param	pdata is the callback function which in our case is the driver
* 		instance.
*
* @return	None
*
* @note		None
*
*******************************************************************************/
void Loop14(void * pdata)
{
	int BytesRead;
	int BytesWritten;
	XUsb *InstancePtr;

	InstancePtr = (XUsb *)pdata;

	/*
	 * Loop date received on EP 1 to EP4
	 */
	while(1) {
		BytesRead = USBRead(InstancePtr, 1, 0, (void *)EpData14,
					MAX_BUF_SIZE);
#ifdef XUSB_EPRW
		printf("Bytes Read 14 = %d \r\n", BytesRead);
#endif
		if(BytesRead == -1) {
			if(RunFlag == 0) {
				printf("Loop14 task, shutdown\n");
			}else {
				printf("Loop14 task, read returned an error\n");
			}
			break;
		}
		BytesWritten = USBWrite(InstancePtr, 4, 0, (void *)EpData14,
					BytesRead);
#ifdef XUSB_EPRW
		printf("Bytes Written 14 = %d \r\n", BytesWritten);
#endif
		if(BytesWritten == -1) {
			if(RunFlag == 0) {
				printf("Loop14 task, shutdown\n");
			}else {
				printf("Loop14 task, read returned an error\n");
			}
			break;
		}
	}
}

/******************************************************************************/
/**
*
* This function is the thread which receives data on end point 2 and 
* transmitting on end point 5
*
* @param	pdata is the callback function which in our case is the driver
* 		instance.
*
* @return	None
*
* @note		None
*
*******************************************************************************/
void Loop25(void * pdata)
{
	int BytesRead;
	int BytesWritten;
	XUsb *InstancePtr;

	InstancePtr = (XUsb *)pdata;

	/*
	 * Loop date received on EP 2 to EP5
	 */
	while (1) {

		BytesRead = USBRead(InstancePtr, 2, 0, (void *)EpData25,
		MAX_BUF_SIZE);
#ifdef XUSB_EPRW
		printf("Bytes Read 25 = %d \r\n", BytesRead);
#endif
		if(BytesRead == -1) {
			if (RunFlag == 0)
			{
				printf("Loop25 task, shutdown\n");
			}else {
				printf("Loop25 task, read returned an error\n");
			}
			break;
		}
		BytesWritten = USBWrite(InstancePtr, 5, 0, (void *)EpData25,
						BytesRead);
#ifdef XUSB_EPRW
		printf("Bytes Written 25 = %d \r\n", BytesWritten);
#endif
		if(BytesWritten == -1) {
			if (RunFlag == 0) {
				printf("Loop25 task, shutdown\n");
			}else {
				printf("Loop25 task, read returned an error\n");
			}
			break;
		}
	}

}

/******************************************************************************/
/**
*
* This function is the thread which receives data on end point 3 and 
* transmitting on end point 6
*
* @param	pdata is the callback function which in our case is the driver
* 		instance.
*
* @return	None
*
* @note		None
*
*******************************************************************************/
void Loop36(void * pdata)
{
	int BytesRead;
	int BytesWritten;
	XUsb *InstancePtr;

	InstancePtr = (XUsb *)pdata;

	/*
	 * Loop date received on EP3 to EP6
	 */
	while(1) {
		BytesRead = USBRead(InstancePtr, 3, 0, (void *)EpData36,
		MAX_BUF_SIZE);
#ifdef XUSB_EPRW
		printf("Bytes Read 36= %d \r\n", BytesRead);
#endif
		if(BytesRead == -1) {
			if(RunFlag == 0) {
				printf("Loop36 task, shutdown\n");
			}else {
				printf("Loop36 task, read returned an error\n");
			}
			break;
		}
		BytesWritten = USBWrite(InstancePtr, 6, 0, (void *)EpData36,
						BytesRead);
#ifdef XUSB_EPRW
		printf("Bytes Written 36 = %d \r\n", BytesWritten);
#endif
		if(BytesWritten == -1) {
			if(RunFlag == 0) {
				printf("Loop36 task, shutdown\n");
			}else {
				printf("Loop36 task, read returned an error\n");
			}
			break;
		}
	}

}
/******************************************************************************/
/**
*
* This function is the thread which waits for the enumeration to be completed 
* and sends the enumeration done event to the main task for starting the user 
* tasks.
*
* @param	pdata is the callback function which in our case is the driver
* 		instance.
*
* @return	None
*
* @note		None
*
*******************************************************************************/
void UsbEnumerated(void *pdata)
{
	XUsb *InstancePtr;

	InstancePtr = (XUsb *)pdata;

	while (1) {
		if (RunFlag == 0) {
			if (InstancePtr->DeviceConfig.CurrentConfiguration == 1)
			{
				UsbEventValue = USB_CONFIG_VALUE;
				sem_post(&UsbEventSem);
				sem_wait(&Epcs[0].Buf0Sem);
			}
		}
	}

}

/******************************************************************************/
/**
*
* This function will shutdown the User Task and flush it's Q's USB Reset, USB
* Suspend, USB Disconnect

* @param	None.
*
* @return	None
*
* @note		None
*
*******************************************************************************/
void ShutdownUserTasks(void)
{

	RunFlag = 0;
	sem_post(&Epcs[0].Buf0Sem);
}

/******************************************************************************/
/**
*
* This function wakes up the pending user tasks.
* @param	InstancePtr is a pointer to the XUsb instance.
*
* @return	None
*
* @note		None
*
*******************************************************************************/
void WakeupAndStop(XUsb *InstancePtr)
{
	int Index;

	for(Index = 1; Index < 8; Index++) {
		/*
		 * Wakeup all pending threads so they can detect the condition
		 */
		if(sem_trywait(&Epcs[Index].Buf0Sem) != 0) {
			/*
			 * If sem_trywait fails, then we need a post, thread is
			 * waiting
			 */
			sem_post(&Epcs[Index].Buf0Sem);
		}
	}
}







