/******************************************************************************
 *
 *       XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
 *       AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
 *       SOLUTIONS FOR XILINX DEVICES. BY PROVIDING THIS DESIGN, CODE,
 *       OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
 *       APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
 *       THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
 *       AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
 *       FOR YOUR IMPLEMENTATION. XILINX EXPRESSLY DISCLAIMS ANY
 *       WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
 *       IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
 *       REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
 *       INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *       FOR A PARTICULAR PURPOSE.
 *
 *       (c) Copyright 2007 Xilinx Inc.
 *       All rights reserved.
 *
 ******************************************************************************/
/******************************************************************************/
/**
 * @file xusb_haldemo.h
 *
 * This file contains the constants, type definitions, variables and function
 * prototypes used in the windows hal application.
 *
 * @note	None.
 *
 * <pre>
 * MODIFICATION HISTORY:
 *
 * Ver   Who  Date     Changes
 * ----- ---- ------------------------------------------------------------------
 * 1.00a hvm  2/12/07 First release
 *
 * </pre>
 *****************************************************************************/

#ifndef  XUSB_HALDEMO_H
#define  XUSB_HALDEMO_H

#ifdef __cplusplus
extern "C" {
#endif

/***************************** Include Files **********************************/

#include "xusb_cp9.h"

/************************** Constant Definitions ******************************/

/*
 * Vendor specific device Id.
 */
#define CLASS_FPGA1		0xFF

/*
 * End point types.
 */
#define EP_CONTROL			0	/**< Control Endpoint */
#define EP_ISOCHRONOUS			1	/**< Isochronous Endpoint */
#define EP_BULK				2	/**< Bulk Endpoint */
#define EP_INTERRUPT			3	/**< Interrupt Endpoint */

/************************** Variable Definitions ******************************/

/*
 * The Usb Descriptors:
 */

USB_STD_DEV_DESC DeviceDescriptor __attribute__ ((aligned (4))) = {
	sizeof (USB_STD_DEV_DESC), 	/* Descriptor Size 18 bytes */
	DEVICE_DESCR, 			/* This is a device descriptor */
	0x02, 				/* USB version */
	CLASS_FPGA1, 			/* Vendor Specific */
	00, 				/* Unused */
	00, 				/* Unused */
	0x40, 				/* Ep0 Max Pkt Size 64 bytes */
	0xfd03, 			/* Vendor Id */
	0x0001,				/* Product Id */
	0x01, 				/* BCD device */
	01, 				/* String Index of manufacturer */
	02, 				/* String Index of product */
	03, 				/* String Index of serial number */
	01				/* Number of configurations */

};

USB_STD_QUAL_DESC QualifierDescriptor __attribute__ ((aligned (4))) = {
	sizeof(USB_STD_QUAL_DESC),
	QUALIFIER_DESCR,
	00,
	02,
	0,
	00,
	00,
	0x40,
	01,
	0
};

FPGA1_CONFIGURATION  __attribute__ ((aligned (4))) HsUsbConfig  = {
	/*
	 * Configuration descriptor
	 */
	{
		sizeof(USB_STD_CFG_DESC),/* size = 9 bytes */
		CONFIG_DESCR,		/* This is a configuration descriptor */
		sizeof(HsUsbConfig),	/* Total size of the configuration LS */
		0x00,			/* Total size MS */
		0x01,			/* Number of interfaces = 1 */
		CONFIGURATION_ONE,	/* Configuration values (1) */
		0x00,			/* Configuration string unused */
		0xc0,			/* Attributes self powered */
		0x01			/* Uses 2ma from USB bus */
	},

	/*
	 * FPGA1 Class interface
	 */
	{
		sizeof(USB_STD_IF_DESC),/* size = 9 bytes */
		INTERFACE_DESCR,	/* Interface descriptor */
		0x00,			/* Interface number 0 */
		0x00,			/* Alternate Set  0 */
		0x07,			/* Number of endpoints = 7 */
		CLASS_FPGA1,		/* Vendor Specific */
		0x00,			/* Inteface subclass = 0 */
		0x00,			/* Protocol is vendor specific */
		0x00			/* Interface is unused */
	},

	/*
	 * Endpoint's 1,2,3 are BULK OUT (input from the PC)
	 */

	/*
	 * Endpoint 1
	 */
	{
		sizeof(USB_STD_EP_DESC),/* Size = 7 */
		ENDPOINT_DESCR,		/* This is an endpoint */
		0x01,			/* Endpoint 1 OUT */
		EP_BULK,		/* Type of endpoint */
		0x00,			/* LS max packet size */
		0x02,			/* MS max packet size */
		0x00			/* NAK rate */
	},

	/*
	 * Endpoint 2
	 */
	{
		sizeof(USB_STD_EP_DESC),/* Size = 7 */
		ENDPOINT_DESCR,		/* This is an endpoint */
		0x02,			/* Endpoint 2 OUT */
		EP_BULK,		/* Type of endpoint */
		0x00,			/* LS max packet size */
		0x02,			/* MS max packet size */
		0x00			/* NAK rate */
	},

	/*
	 * Endpoint 3
	 */
	{
		sizeof(USB_STD_EP_DESC), /* Size = 7 */
		ENDPOINT_DESCR,		/* This is an endpoint */
		0x03,			/* Endpoint 3 OUT */
		EP_BULK,		/* Type of endpoint */
		0x00,			/* LS max packet size */
		0x02,			/* MS max packet size */
		0x00			/* NAK rate */
	},

	/*
	 * Endpoints 4,5,6 are BULK IN (to the PC)
	 */

	/*
	 * Endpoint 4
	 */
	{
		sizeof(USB_STD_EP_DESC), /* Size = 7 */
		ENDPOINT_DESCR,		/* This is an endpoint */
		0x84,			/* Endpoint 4 IN */
		EP_BULK,		/* Type of endpoint */
		0x00,			/* LS max packet size */
		0x02,			/* MS max packet size */
		0x00			/* NAK rate */
	},

	/*
	 * Endpoint 5
	 */
	{
		sizeof(USB_STD_EP_DESC),/* Size = 7 */
		ENDPOINT_DESCR,		/* This is an endpoint */
		0x85,			/* Endpoint 5 IN */
		EP_BULK,		/* Type of endpoint */
		0x00,			/* LS max packet size */
		0x02,			/* MS max packet size */
		0x00			/* NAK rate */
	},

	/*
	 * Endpoint 6
	 */
	{
		sizeof(USB_STD_EP_DESC),/* Size = 7 */
		ENDPOINT_DESCR,		/* This is an endpoint */
		0x86,			/* Endpoint 6 IN */
		EP_BULK,		/* Type of endpoint */
		0x00,			/* LS max packet size */
		0x02,			/* MS max packet size */
		0x00			/* NAK rate */
	},

	/*
	 * Endpoint 7 is the Interrupt IN endpoint
	 */

	{
		sizeof(USB_STD_EP_DESC), /* Size = 7 */
		ENDPOINT_DESCR,		/* This is an endpoint */
		0x87,			/* Endpoint 7 IN */
		EP_INTERRUPT,		/* Type of endpoint */
		0x10,			/* LS max packet size */
		0x00,			/* MS max packet size */
		0x01			/* NAK rate */
	}
};


FPGA1_CONFIGURATION  __attribute__ ((aligned (4))) FsUsbConfig  = {

	/*
	 * Configuration descriptor
	 */
	{
		sizeof(USB_STD_CFG_DESC),/* size = 9 bytes */
		CONFIG_DESCR,		/* This is a configuration descriptor */
		sizeof(FsUsbConfig),	/* Total size of the configuration LS */
		0x00,			/* Total size MS */
		0x01,			/* Number of interfaces = 1 */
		CONFIGURATION_ONE,	/* Configuration values (1) */
		0x00,			/* Configuration string unused */
		0xc0,			/* Attributes self powered */
		0x01			/* Uses 2ma from USB bus */
	},

	/*
	 * FPGA1 Class interface
	 */
	{
		sizeof(USB_STD_IF_DESC),/* size = 9 bytes */
		INTERFACE_DESCR,	/* Interface descriptor */
		0x00,			/* Interface number 0 */
		0x00,			/* Alternate Set  0 */
		0x07,			/* Number of endpoints = 7 */
		CLASS_FPGA1,		/* Vendor Specific */
		0x00,			/* Inteface subclass = 0 */
		0x00,			/* Protocol is vendor specific */
		0x00			/* Interface is unused */
	},

	/*
	 * Endpoint's 1,2,3 are BULK OUT (input from the PC)
	 */

	/*
	 * Endpoint 1
	 */
	{
		sizeof(USB_STD_EP_DESC),/* Size = 7 */
		ENDPOINT_DESCR,		/* This is an endpoint */
		0x01,			/* Endpoint 1 OUT */
		EP_BULK,		/* Type of endpoint */
		0x40,			/* LS max packet size */
		0x00,			/* MS max packet size */
		0x00			/* NAK rate */
	},

	/*
	 * Endpoint 2
	 */
	{
		sizeof(USB_STD_EP_DESC), /* Size = 7 */
		ENDPOINT_DESCR,		/* This is an endpoint */
		0x02,			/* Endpoint 2 OUT */
		EP_BULK,		/* Type of endpoint */
		0x40,			/* LS max packet size */
		0x00,			/* MS max packet size */
		0x00			/* NAK rate */
	},

	/*
	 * Endpoint 3
	 */
	{
		sizeof(USB_STD_EP_DESC),/* Size = 7 */
		ENDPOINT_DESCR,		/* This is an endpoint */
		0x03,			/* Endpoint 3 OUT */
		EP_BULK,		/* Type of endpoint */
		0x40,			/* LS max packet size */
		0x00,			/* MS max packet size */
		0x00 			/* NAK rate */
	},

	/*
	 * Endpoints 4,5,6 are BULK IN (to the PC)
	 */

	/*
	 * Endpoint 4
	 */
	{
		sizeof(USB_STD_EP_DESC),/* Size = 7 */
		ENDPOINT_DESCR,		/* This is an endpoint */
		0x84,			/* Endpoint 4 IN */
		EP_BULK,		/* Type of endpoint */
		0x40,			/* LS max packet size */
		0x00,			/* MS max packet size */
		0x00 			/* NAK rate */
	},

	/*
	 * Endpoint 5
	 */
	{
		sizeof(USB_STD_EP_DESC),/* Size = 7 */
		ENDPOINT_DESCR,		/* This is an endpoint */
		0x85,			/* Endpoint 5 IN */
		EP_BULK,		/* Type of endpoint */
		0x40,			/* LS max packet size */
		0x00,			/* MS max packet size */
		0x00 			/* NAK rate */
	},

	/*
	 * Endpoint 6
	 */
	{
		sizeof(USB_STD_EP_DESC), /* Size = 7 */
		ENDPOINT_DESCR,		/* This is an endpoint */
		0x86,			/* Endpoint 6 IN */
		EP_BULK,		/* Type of endpoint */
		0x40,			/* LS max packet size */
		0x00,			/* MS max packet size */
		0x00			/* NAK rate */
	},

	/*
	 * Endpoint 7 is the Interrupt IN endpoint
	 */
	{
		sizeof(USB_STD_EP_DESC), /* Size = 7 */
		ENDPOINT_DESCR,		/* This is an endpoint */
		0x87,			/* Endpoint 7 IN */
		EP_INTERRUPT,		/* Type of endpoint */
		0x10,			/* LS max packet size */
		0x00,			/* MS max packet size */
		0xff			/* NAK rate */
	}

};

USB_STD_STRING_DESC LangId __attribute__ ((aligned (4))) = {
	/*
	 * Language ID codes
	 */
	4, STRING_DESCR, {0x0904}
};

USB_STD_STRING_MAN_DESC Manufacturer __attribute__ ((aligned (4))) = {
	/*
	 * Manufacturer String
	 */
	sizeof(USB_STD_STRING_MAN_DESC), STRING_DESCR,
	{'X',0,'I',0,'L',0,'I',0,'N',0,'X',0,' ',0}
};

USB_STD_STRING_PS_DESC ProductString __attribute__ ((aligned (4))) = {
	/*
	 * Product ID String
	 */
	sizeof(USB_STD_STRING_PS_DESC), STRING_DESCR,
	{'F',0,'P',0,'G',0,'A',0,'0',0}
};

USB_STD_STRING_SN_DESC SerialNumber __attribute__ ((aligned (4))) = {
	/*
	 * Product ID String
	 */
	sizeof(USB_STD_STRING_SN_DESC), STRING_DESCR,
	{'0',0,'0',0,'0',0,'0',0,'0',0,'0',0,'0',0,'1',0,
	'7',0,'1',0,'5',0,'0',0,'4',0,'2',0,'6',0,'2',0,
	'0',0,'0',0,'5',0,'7',0,'5',0}
};


/************************** Function Prototypes *******************************/

void  	InitUsbInterface(XUsb *InstancePtr);
void 	UsbIfIntrHandler(void *CallBackRef, u32 IntrStatus);
void 	EpIntrHandler(void *CallBackRef, u8 EpNum, u32 IntrStatus);
void 	Ep0IntrHandler(void *CallBackRef, u8 EpNum, u32 IntrStatus);
void 	Ep1IntrHandler(void *CallBackRef, u8 EpNum, u32 IntrStatus);
void 	Ep2IntrHandler(void *CallBackRef, u8 EpNum, u32 IntrStatus);
void 	Ep3IntrHandler(void *CallBackRef, u8 EpNum, u32 IntrStatus);
void 	Ep4IntrHandler(void *CallBackRef, u8 EpNum, u32 IntrStatus);
void 	Ep5IntrHandler(void *CallBackRef, u8 EpNum, u32 IntrStatus);
void 	Ep6IntrHandler(void *CallBackRef, u8 EpNum, u32 IntrStatus);
void 	Ep7IntrHandler(void *CallBackRef, u8 EpNum, u32 IntrStatus);

static int SetupInterruptSystem(XUsb *InstancePtr);
int USBRead(XUsb *InstancePtr, int EpNum, int NoBlock, char *BufPtr, 
		int Length);
int USBWrite(XUsb *InstancePtr, int EpNum, int NoBlock, char *BufPtr, 
		int Length);

#ifdef __cplusplus
}
#endif

#endif /* XUSB_HALDEMO_H */


















