//******************************************************************************
// *
// *       XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
// *       AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
// *       SOLUTIONS FOR XILINX DEVICES. BY PROVIDING THIS DESIGN, CODE,
// *       OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
// *       APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
// *       THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
// *       AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
// *       FOR YOUR IMPLEMENTATION. XILINX EXPRESSLY DISCLAIMS ANY
// *       WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
// *       IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
// *       REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
// *       INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
// *       FOR A PARTICULAR PURPOSE.
// *
// *		Copyright(C) 2005 by xilinx, Inc. All rights reserved.              
// *
// *       (c) Copyright 2007 Xilinx Inc.
// *       All rights reserved.
//*
// ******************************************************************************/

#ifndef __XILINXDD_H__
#define __XILINXDD_H__

// Memory allocation pool tag
#define XILINXDD_POOL_TAG 'eerv'

// Make all pool allocations tagged
#undef ExAllocatePool
#define ExAllocatePool(type, size) \
    ExAllocatePoolWithTag(type, size, XILINXDD_POOL_TAG);

// queue start io callback
typedef VOID (*PXILINXDD_QUEUE_STARTIO)(
    IN PDEVICE_OBJECT   DeviceObject,
    IN PIRP             Irp
    );

// irp queue type definition
typedef struct _XILINXDD_QUEUE
{
    PXILINXDD_QUEUE_STARTIO StartIoRoutine;
    PDEVICE_OBJECT  DeviceObject;
    LIST_ENTRY      IrpQueue;
    KSPIN_LOCK      QueueLock;
    PIRP            CurrentIrp;
    KEVENT          StopEvent;
    LONG            StallCount;
    NTSTATUS        ErrorStatus;
    KDPC            xilinxddStartIoDpc;
    BOOLEAN         bUsexilinxddStartIoDpc;
} XILINXDD_QUEUE, *PXILINXDD_QUEUE;

// cancel-safe irp list type definition
typedef struct _XILINXDD_LIST
{
    PDEVICE_OBJECT  DeviceObject;
    LIST_ENTRY      IrpList;
    KSPIN_LOCK      ListLock;
    NTSTATUS        ErrorStatus;
} XILINXDD_LIST, *PXILINXDD_LIST;

// stall IRP list to syncronize Pnp, Power with
// the rest of IO
typedef struct _XILINXDD_IO_LOCK
{
    PDEVICE_OBJECT  DeviceObject;       // our device object
    KEVENT          StallCompleteEvent; // io stalled event
    LIST_ENTRY      StallIrpList;       // stalled irps
    KSPIN_LOCK      IoLock;             // spin lock to syncronize io with stall/unstall
    LONG            StallCount;         // number of times stall was requested
    LONG            ActiveIrpCount;     // number of oustanding, not-stalledIRPs
    NTSTATUS        ErrorStatus;
    PIRP            CurrentIrp;         // used by unstall code
} XILINXDD_IO_LOCK, *PXILINXDD_IO_LOCK;

// global (per driver) data block
typedef struct _XILINXDD_DATA
{
    UNICODE_STRING      RegistryPath;           // saved registry path
    ULONG               InstanceCount;          // FDO instance count
    USHORT              WdmVersion;             // os version
} XILINXDD_DATA, *PXILINXDD_DATA;

extern XILINXDD_DATA g_Data;

// PnP states
typedef enum _XILINXDD_PNP_STATE
{
    PnpStateNotStarted = 0,
    PnpStateStarted,
    PnpStateStopPending,
    PnpStateStopped,
    PnpStateRemovePending,
    PnpStateRemoved,
    PnpStateSurpriseRemoved
} XILINXDD_PNP_STATE;


// bulk transfer context type
typedef struct _XILINXDD_IO_CONTEXT
{
    struct _XILINXDD_DEVICE_EXTENSION*   DeviceExtension;

    PUCHAR              NextVirtualAddress;
    ULONG               RemainingLength;
    ULONG               TotalTransferCount;
    PMDL                Mdl;
    PURB                Urb;
} XILINXDD_IO_CONTEXT, *PXILINXDD_IO_CONTEXT;

// The device extension for the device object
typedef struct _XILINXDD_DEVICE_EXTENSION
{
    PDEVICE_OBJECT          DeviceObject;           // pointer to the    DeviceObject
    PDEVICE_OBJECT          PhysicalDeviceObject;   // underlying PDO
    PDEVICE_OBJECT          LowerDeviceObject;      // top of the device stack
    LONG                    RemoveCount;            // 1-based reference count
    KEVENT                  RemoveEvent;            // event to sync device  removal
    XILINXDD_IO_LOCK IoLock;                 // misc io lock
    XILINXDD_PNP_STATE    PnpState;               // PnP state variable
    XILINXDD_PNP_STATE    PreviousPnpState;       // Previous PnP state variable
    UNICODE_STRING          InterfaceName;          // name returned from  IoRegisterDeviceInterface
    SYSTEM_POWER_STATE      SystemPowerState;       // current system powerstate
    DEVICE_POWER_STATE      DevicePowerState;       // current device powerstate
    DEVICE_CAPABILITIES     DeviceCaps;             // device capabilities usedto get S to D mappings
    PIRP                    SystemPowerIrp;         // Storage for system IRP  during power transitions
    PIRP                    DevicePowerIrp;         // Storage for device IRP  during power transitions
    PIO_WORKITEM            PowerWorkItem;          // Work item used for power   transitions
    LONG                    bPowerStop;             // Flag to signal that powercode stopped io
    LONG                    OpenHandleCount;
    PUSB_CONFIGURATION_DESCRIPTOR   ConfigDescriptor;   // configuration descriptor
    USBD_CONFIGURATION_HANDLE       ConfigHandle;       // configuration handle
    PUSBD_INTERFACE_INFORMATION     InterfaceInformation;
    PIRP                            Pipe7Irp;
    PURB                            Pipe7Urb;
    UCHAR                           Pipe7Data[16];
    LONG                            Pipe7Armed;
} XILINXDD_DEVICE_EXTENSION, *PXILINXDD_DEVICE_EXTENSION;

#ifdef __cplusplus
extern "C"
{
#endif

NTSTATUS __stdcall DriverEntry(
    IN  PDRIVER_OBJECT  DriverObject,
    IN  PUNICODE_STRING RegistryPath
    );

NTSTATUS __stdcall xilinxddAddDevice(
    IN  PDRIVER_OBJECT  DriverObject,
    IN  PDEVICE_OBJECT  PhysicalDeviceObject
    );

VOID __stdcall xilinxddUnload(
    IN  PDRIVER_OBJECT  DriverObject
    );

NTSTATUS __stdcall xilinxddPnpDispatch(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    );

NTSTATUS __stdcall xilinxddPowerDispatch(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    );

NTSTATUS __stdcall xilinxddDeviceIoControlDispatch(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    );

NTSTATUS __stdcall xilinxddCreateDispatch(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    );

LONG
xilinxdd_PipeWIthName(
                    IN PDEVICE_OBJECT  DeviceObject,
                    IN PUNICODE_STRING FileName
                    );

NTSTATUS __stdcall xilinxddCloseDispatch(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    );

NTSTATUS __stdcall xilinxddReadDispatch(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    );

NTSTATUS __stdcall xilinxddWriteDispatch(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    );

NTSTATUS __stdcall xilinxddCleanupDispatch(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    );

NTSTATUS __stdcall xilinxddSystemControlDispatch(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    );

////////////////////////////////////////////////////////////////////////////////
///////////////////
// Queue Functions
////////////////////////////////////////////////////////////////////////////////
///////////////////

VOID xilinxddInitializeQueue(
    IN  PXILINXDD_QUEUE    QueueExtension,
    IN  PXILINXDD_QUEUE_STARTIO     StartIoRoutine,
    IN  PDEVICE_OBJECT      DeviceObject,
    IN  BOOLEAN             bUsexilinxddStartIoDpc
    );

NTSTATUS xilinxddQueueIrp(
    IN  PXILINXDD_QUEUE    QueueExtension,
    IN  PIRP                Irp
    );

VOID xilinxddStartNext(
    IN  PXILINXDD_QUEUE    QueueExtension
    );

VOID xilinxddFlushQueue(
    IN  PXILINXDD_QUEUE    QueueExtension,
    IN  PFILE_OBJECT        FileObject
    );

VOID xilinxddInvalidateQueue(
    IN  PXILINXDD_QUEUE    Queue,
    IN  NTSTATUS                  ErrorStatus
    );

VOID xilinxddPauseQueue(
    IN  PXILINXDD_QUEUE    QueueExtension
    );

VOID xilinxddRestartQueue(
    IN  PXILINXDD_QUEUE    QueueExtension
    );

VOID xilinxddStartIoDpc(
    IN  PKDPC       Dpc,
    IN  PVOID       Context,
    IN  PVOID       Unused1,
    IN  PVOID       Unused2
    );

VOID xilinxddQueueCancelRoutine(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    );

////////////////////////////////////////////////////////////////////////////////
///////////////////
// List functions
////////////////////////////////////////////////////////////////////////////////
///////////////////

VOID xilinxddInitializeList(
    IN  PXILINXDD_LIST   List,
    IN  PDEVICE_OBJECT          DeviceObject
    );

NTSTATUS xilinxddInsertHead(
    IN  PXILINXDD_LIST   List,
    IN  PIRP                Irp
    );

NTSTATUS xilinxddInsertTail(
    IN  PXILINXDD_LIST   List,
    IN  PIRP                Irp
    );

PIRP xilinxddRemoveHead(
    IN  PXILINXDD_LIST   List
    );

PIRP xilinxddRemoveTail(
    IN  PXILINXDD_LIST   List
    );

VOID xilinxddFlushList(
    IN  PXILINXDD_LIST   List,
    IN  PFILE_OBJECT        FileObject
    );

VOID xilinxddInvalidateList(
    IN  PXILINXDD_LIST   List,
    IN  NTSTATUS                ErrorStatus
    );

VOID xilinxddListCancelRoutine(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    );

////////////////////////////////////////////////////////////////////////////////
///////////////////
// XILINXDD_IO_LOCK
////////////////////////////////////////////////////////////////////////////////
///////////////////

VOID xilinxddInitializeIoLock(
    IN  PXILINXDD_IO_LOCK    IoLock,
    IN  PDEVICE_OBJECT              DeviceObject
    );

NTSTATUS xilinxddCheckIoLock(
    IN  PXILINXDD_IO_LOCK    IoLock,
    IN  PIRP                        Irp
    );

VOID xilinxddPendingIoCancelRoutine(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    );

NTSTATUS xilinxddIncrementIoCount(
    IN  PXILINXDD_IO_LOCK    IoLock
    );

VOID xilinxddDecrementIoCount(
    IN  PXILINXDD_IO_LOCK    IoLock
    );

VOID xilinxddLockIo(
    IN  PXILINXDD_IO_LOCK    IoLock
    );

VOID xilinxddWaitForStopIo(
    IN  PXILINXDD_IO_LOCK    IoLock
    );

VOID xilinxddUnlockIo(
    IN  PXILINXDD_IO_LOCK    IoLock
    );

VOID xilinxddFlushPendingIo(
    IN  PXILINXDD_IO_LOCK    IoLock,
    IN  PFILE_OBJECT                FileObject
    );

VOID xilinxddInvalidateIo(
    IN  PXILINXDD_IO_LOCK    IoLock,
    IN  NTSTATUS                    ErrorStatus
    );
#ifdef __cplusplus
}
#endif

BOOLEAN xilinxddIsStoppable(
    IN  PXILINXDD_DEVICE_EXTENSION    DeviceExtension
    );

BOOLEAN xilinxddIsRemovable(
    IN  PXILINXDD_DEVICE_EXTENSION    DeviceExtension
    );

BOOLEAN xilinxddAcquireRemoveLock(
    IN  PXILINXDD_DEVICE_EXTENSION    DeviceExtension
    );

VOID xilinxddReleaseRemoveLock(
    IN  PXILINXDD_DEVICE_EXTENSION    DeviceExtension
    );

VOID xilinxddWaitForSafeRemove(
    IN  PXILINXDD_DEVICE_EXTENSION    DeviceExtension
    );

NTSTATUS xilinxddSubmitIrpSync(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    );

VOID xilinxddStallQueues(
    IN  PXILINXDD_DEVICE_EXTENSION DeviceExtension
    );

VOID xilinxddRestartQueues(
    IN  PXILINXDD_DEVICE_EXTENSION DeviceExtension
    );

VOID xilinxddFlushQueues(
    IN  PXILINXDD_DEVICE_EXTENSION   DeviceExtension,
    IN  PFILE_OBJECT            FileObject
    );

////////////////////////////////////////////////////////////////////////////////
///////////////////
// Pnp
////////////////////////////////////////////////////////////////////////////////
///////////////////

// define this PnP IRP.  This IRP is only defined in ntddk.h normally
#if !defined(IRP_MN_QUERY_LEGACY_BUS_INFORMATION)
#define IRP_MN_QUERY_LEGACY_BUS_INFORMATION     0x18
#endif // IRP_MN_QUERY_LEGACY_BUS_INFORMATION

NTSTATUS xilinxddStartDevice(
    IN  PXILINXDD_DEVICE_EXTENSION    DeviceExtension,
    IN  PIRP                            Irp
    );

NTSTATUS xilinxddFreeResources(
    IN  PXILINXDD_DEVICE_EXTENSION DeviceExtension
    );

NTSTATUS xilinxddGetDeviceCapabilities(
    IN  PXILINXDD_DEVICE_EXTENSION    DeviceExtension
    );

////////////////////////////////////////////////////////////////////////////////
///////////////////
// Power
////////////////////////////////////////////////////////////////////////////////
///////////////////

typedef struct _XILINXDD_POWER_IRP_CONTEXT
{
    PKEVENT         Event;
    NTSTATUS        Status;
} XILINXDD_POWER_IRP_CONTEXT, *PXILINXDD_POWER_IRP_CONTEXT;


#ifdef __cplusplus
extern "C" {
#endif // __cplusplus

NTSTATUS xilinxddSystemPowerDispatch(
    IN  PXILINXDD_DEVICE_EXTENSION        DeviceExtension,
    IN  PIRP                        Irp
    );

NTSTATUS xilinxddSystemPowerCompletionRoutine(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp,
    IN  PVOID           Context
    );

NTSTATUS xilinxddDeviceQueryPowerCompletionRoutine(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp,
    IN  PVOID           Context
    );

NTSTATUS xilinxddDeviceSetPowerCompletionRoutine(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp,
    IN  PVOID           Context
    );

VOID xilinxddDevicePowerCompleteCallback(
    IN  PDEVICE_OBJECT      DeviceObject,
    IN  UCHAR               MinorFunction,
    IN  POWER_STATE         PowerState,
    IN  PVOID               Context,
    IN  PIO_STATUS_BLOCK    IoStatus
    );

#ifdef __cplusplus
}
#endif // __cplusplus

NTSTATUS xilinxddGetDevicePowerState(
    IN  PXILINXDD_DEVICE_EXTENSION        DeviceExtension,
    IN  SYSTEM_POWER_STATE              SystemState,
    OUT PDEVICE_POWER_STATE             DeviceState
    );

VOID xilinxddPowerDownPreparation(
    IN  PXILINXDD_DEVICE_EXTENSION        DeviceExtension,
    IN  PIO_WORKITEM_ROUTINE                Callback
    );

VOID xilinxddPowerDownPrepCallback(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PVOID           Context
    );

VOID xilinxddPowerD0PrepCallback(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PVOID           Context
    );

VOID xilinxddPowerUpCallback(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PVOID           Context
    );

VOID xilinxddRequestDevicePowerIrpCallback(
    IN  PDEVICE_OBJECT      DeviceObject,
    IN  UCHAR               MinorFunction,
    IN  POWER_STATE         PowerState,
    IN  PVOID               Context,
    IN  PIO_STATUS_BLOCK    IoStatus
    );

NTSTATUS xilinxddRequestDevicePowerIrp(
    IN  PXILINXDD_DEVICE_EXTENSION        DeviceExtension,
    IN  DEVICE_POWER_STATE             DeviceState
    );


////////////////////////////////////////////////////////////////////////////////
///////////////////
// Registry
////////////////////////////////////////////////////////////////////////////////
///////////////////

PVOID xilinxddRegQueryValueKey(
    IN  HANDLE  RegKeyHandle,
    IN  PWSTR   SubKeyName,
    IN  PWSTR   ValueName,
    OUT PULONG  Length
    );

VOID xilinxddRegEnumerateKeys(
    IN  HANDLE RegKeyHandle
    );

VOID xilinxddRegEnumerateValueKeys(
    IN  HANDLE  RegKeyHandle
    );

////////////////////////////////////////////////////////////////////////////////
///////////////////
// Debug
////////////////////////////////////////////////////////////////////////////////
///////////////////

// definition of debug levels

#define DBG_NONE            0
#define DBG_ERR             1
#define DBG_WARN            2
#define DBG_TRACE           3
#define DBG_INFO            4
#define DBG_VERB            5

#ifdef XILINXDD_WMI_TRACE

/*
tracepdb -f objchk_wxp_x86\i386\xilinxdd.pdb -p C:\xilinxdd
SET TRACE_FORMAT_SEARCH_PATH=C:\xilinxdd

tracelog -start xilinxdd -guid xilinxdd.ctl -f xilinxdd.log -flags 0x7FFFFFFF
-level 5
tracelog -stop xilinxdd

tracefmt -o xilinxdd.txt -f xilinxdd.log
*/

#define WPP_AREA_LEVEL_LOGGER(Area,Lvl)           WPP_LEVEL_LOGGER(Area)
#define WPP_AREA_LEVEL_ENABLED(Area,Lvl)          (WPP_LEVEL_ENABLED(Area) &&
WPP_CONTROL(WPP_BIT_##Area).Level >= Lvl)

#define WPP_CONTROL_GUIDS \
    WPP_DEFINE_CONTROL_GUID(xilinxdd,(E483AAED,C2BC,4C91,AD4C,D9625611E762), \
        WPP_DEFINE_BIT(DBG_GENERAL)                 /* bit  0 = 0x00000001 */ \
        WPP_DEFINE_BIT(DBG_PNP)                     /* bit  1 = 0x00000002 */ \
        WPP_DEFINE_BIT(DBG_POWER)                   /* bit  2 = 0x00000004 */ \
        WPP_DEFINE_BIT(DBG_COUNT)                   /* bit  3 = 0x00000008 */ \
        WPP_DEFINE_BIT(DBG_CREATECLOSE)             /* bit  4 = 0x00000010 */ \
        WPP_DEFINE_BIT(DBG_WMI)                     /* bit  5 = 0x00000020 */ \
        WPP_DEFINE_BIT(DBG_UNLOAD)                  /* bit  6 = 0x00000040 */ \
        WPP_DEFINE_BIT(DBG_IO)                      /* bit  7 = 0x00000080 */ \
        WPP_DEFINE_BIT(DBG_INIT)                    /* bit  8 = 0x00000100 */ \
        WPP_DEFINE_BIT(DBG_09)                      /* bit  9 = 0x00000200 */ \
        WPP_DEFINE_BIT(DBG_10)                      /* bit 10 = 0x00000400 */ \
        WPP_DEFINE_BIT(DBG_11)                      /* bit 11 = 0x00000800 */ \
        WPP_DEFINE_BIT(DBG_12)                      /* bit 12 = 0x00001000 */ \
        WPP_DEFINE_BIT(DBG_13)                      /* bit 13 = 0x00002000 */ \
        WPP_DEFINE_BIT(DBG_14)                      /* bit 14 = 0x00004000 */ \
        WPP_DEFINE_BIT(DBG_15)                      /* bit 15 = 0x00008000 */ \
        WPP_DEFINE_BIT(DBG_16)                      /* bit 16 = 0x00010000 */ \
        WPP_DEFINE_BIT(DBG_17)                      /* bit 17 = 0x00020000 */ \
        WPP_DEFINE_BIT(DBG_18)                      /* bit 18 = 0x00040000 */ \
        WPP_DEFINE_BIT(DBG_19)                      /* bit 19 = 0x00080000 */ \
        WPP_DEFINE_BIT(DBG_20)                      /* bit 20 = 0x00100000 */ \
        WPP_DEFINE_BIT(DBG_21)                      /* bit 21 = 0x00200000 */ \
        WPP_DEFINE_BIT(DBG_22)                      /* bit 22 = 0x00400000 */ \
        WPP_DEFINE_BIT(DBG_23)                      /* bit 23 = 0x00800000 */ \
        WPP_DEFINE_BIT(DBG_24)                      /* bit 24 = 0x01000000 */ \
        WPP_DEFINE_BIT(DBG_25)                      /* bit 25 = 0x02000000 */ \
        WPP_DEFINE_BIT(DBG_26)                      /* bit 26 = 0x04000000 */ \
        WPP_DEFINE_BIT(DBG_27)                      /* bit 27 = 0x08000000 */ \
        WPP_DEFINE_BIT(DBG_28)                      /* bit 28 = 0x10000000 */ \
        WPP_DEFINE_BIT(DBG_29)                      /* bit 29 = 0x20000000 */ \
        WPP_DEFINE_BIT(DBG_30)                      /* bit 30 = 0x40000000 */ \
        WPP_DEFINE_BIT(DBG_31)                      /* bit 31 = 0x80000000 */ \
        )

__inline VOID xilinxddDumpIrp(
    IN PIRP Irp
    )
{
}

__inline PCHAR SystemPowerStateString(
    IN  SYSTEM_POWER_STATE  SystemState
    )
{
    return "";
}

__inline PCHAR DevicePowerStateString(
    IN  DEVICE_POWER_STATE  DeviceState
    )
{
    return "";
}

#else

// definition of debug areas

#define DBG_GENERAL         (1 << 0)
#define DBG_PNP             (1 << 1)
#define DBG_POWER           (1 << 2)
#define DBG_COUNT           (1 << 3)
#define DBG_CREATECLOSE     (1 << 4)
#define DBG_WMI             (1 << 5)
#define DBG_UNLOAD          (1 << 6)
#define DBG_IO              (1 << 7)
#define DBG_INIT            (1 << 8)

#define DBG_ALL             0xFFFFFFFF

#if DBG

VOID xilinxddDebugPrint(
    IN ULONG    Area,
    IN ULONG    Level,
    IN PCCHAR   Format,
    IN          ...
    );

VOID xilinxddDumpIrp(
    IN PIRP Irp
    );

PCHAR IrpMajorFunctionString(
    IN  UCHAR MajorFunction
    );

PCHAR PnPMinorFunctionString(
    IN  UCHAR   MinorFunction
    );

PCHAR PowerMinorFunctionString(
    IN  UCHAR   MinorFunction
    );

PCHAR SystemPowerStateString(
    IN  SYSTEM_POWER_STATE  SystemState
    );

PCHAR DevicePowerStateString(
    IN  DEVICE_POWER_STATE  DeviceState
    );

PCHAR WMIMinorFunctionString (
    IN  UCHAR MinorFunction
    );

#else	// !DBG

__inline VOID xilinxddDebugPrint(
    IN ULONG    Area,
    IN ULONG    Level,
    IN PCCHAR   Format,
    IN          ...
    )
{
}

#define xilinxddDumpIrp(Irp)
#define IrpMajorFunctionString(MajorFunction)
#define PnPMinorFunctionString(MinorFunction)
#define PowerMinorFunctionString(MinorFunction)
#define SystemPowerStateString(SystemState) 0
#define DevicePowerStateString(DeviceState) 0
#define WMIMinorFunctionString(MinorFunction)
#define xilinxddTdiDumpAddress(Address)

#endif	// DBG
#endif	// XILINXDD_WMI_TRACE

NTSTATUS xilinxddSubmitUrbSynch(
    IN  PXILINXDD_DEVICE_EXTENSION   DeviceExtension,
    IN  PURB                        Urb
    );

NTSTATUS xilinxddResetPipe(
    IN  PXILINXDD_DEVICE_EXTENSION   DeviceExtension,
    IN  USBD_PIPE_HANDLE                    PipeHandle
    );

NTSTATUS xilinxddAbortPipe(
    IN  PXILINXDD_DEVICE_EXTENSION   DeviceExtension,
    IN  USBD_PIPE_HANDLE                    PipeHandle
    );

NTSTATUS xilinxddResetDevice(
    IN  PXILINXDD_DEVICE_EXTENSION   DeviceExtension
    );

NTSTATUS xilinxddSelectAlternateInterface(
    IN  PXILINXDD_DEVICE_EXTENSION   DeviceExtension,
    IN  ULONG                               AlternateSetting
    );

NTSTATUS xilinxddPipeOutIo(
    IN  PXILINXDD_DEVICE_EXTENSION          DeviceExtension,
    IN  PIRP                                Irp,
    IN  PUSBD_PIPE_INFORMATION              PipeInformation
    );

NTSTATUS xilinxddPipeOutIoCompletionRoutine(
    IN  PDEVICE_OBJECT DeviceObject,
    IN  PIRP           Irp,
    IN  PVOID          Context
    );

NTSTATUS xilinxddPipeInIo(
    IN  PXILINXDD_DEVICE_EXTENSION   DeviceExtension,
    IN  PIRP                                Irp,
    IN  PUSBD_PIPE_INFORMATION              PipeInformation
    );

NTSTATUS xilinxddPipeInIoCompletionRoutine(
    IN  PDEVICE_OBJECT DeviceObject,
    IN  PIRP           Irp,
    IN  PVOID          Context
    );

NTSTATUS
Vreelindd_ResetPipe(
                 IN PDEVICE_OBJECT         DeviceObject,
                 IN PUSBD_PIPE_INFORMATION PipeInfo
                 );

NTSTATUS
Vreelindd_ResetDevice(
                   IN PDEVICE_OBJECT DeviceObject
                   );

NTSTATUS
Vreelindd_GetPortStatus(
                     IN PDEVICE_OBJECT DeviceObject,
                     IN OUT PULONG     PortStatus
                     );
#endif  // __XILINXDD_H__

